@@ -203,6 +203,22 @@ class Redis(object):
             transaction,
             shard_hint)
 
+    def transaction(self, func, *watches, **kwargs):
+        """
+        Convenience method for executing the callable `func` as a transaction
+        while watching all keys specified in `watches`. The 'func' callable
+        should expect a single arguement which is a Pipeline object.
+        """
+        shard_hint = kwargs.pop('shard_hint', None)
+        with self.pipeline(True, shard_hint) as pipe:
+            while 1:
+                try:
+                    pipe.watch(*watches)
+                    func(pipe)
+                    return pipe.execute()
+                except WatchError:
+                    continue
+
     def lock(self, name, timeout=None, sleep=0.1):
         """
         Return a new Lock object using key ``name`` that mimics
@@ -1196,6 +1212,12 @@ class Pipeline(Redis):
         self.watching = False
         self.reset()
 
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exc_type, exc_value, traceback):
+        self.reset()
+
     def reset(self):
         self.command_stack = []
         # make sure to reset the connection state in the event that we were
@@ -1334,8 +1356,14 @@ class Pipeline(Redis):
             execute = self._execute_transaction
         else:
             execute = self._execute_pipeline
-        conn = self.connection or \
-                   self.connection_pool.get_connection('MULTI', self.shard_hint)
+
+        conn = self.connection
+        if not conn:
+            conn = self.connection_pool.get_connection('MULTI', self.shard_hint)
+            # assign to self.connection so reset() releases the connection
+            # back to the pool after we're done
+            self.connection = conn
+
         try:
             return execute(conn, stack)
         except ConnectionError:
