@@ -381,6 +381,7 @@ struct sharedObjectsStruct {
 /* Global server state structure */
 struct redisServer {
     pthread_t mainthread;
+    int arch_bits;
     int port;
     char *bindaddr;
     char *unixsocket;
@@ -400,6 +401,7 @@ struct redisServer {
     /* Fast pointers to often looked up command */
     struct redisCommand *delCommand, *multiCommand;
     list *slaves, *monitors;
+    redisClient *current_client; /* Current client, only used on crash report */
     char neterr[ANET_ERR_LEN];
     aeEventLoop *el;
     int cronloops;              /* number of times the cron function run */
@@ -1072,4 +1074,6 @@ void *malloc(size_t size) __attribute__ ((deprecated));
 void *realloc(void *ptr, size_t size) __attribute__ ((deprecated));
 #endif
 
+void redisLogObjectDebugInfo(robj *o);
+
 #endif
